/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.dao;

import com.geekcap.modeler.model.DomainModel;
import com.geekcap.modeler.model.DomainObject;
import com.geekcap.modeler.model.Field;
import com.geekcap.modeler.model.Operation;
import com.geekcap.modeler.model.OperationParameter;
import com.geekcap.modeler.model.Relationship;
import com.geekcap.modeler.model.Stereotype;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;

/**
 * Builds domain objects from a UML 1.4 XMI file (version 1.2)
 *
 * @author shaines
 */
public class Xmi14DomainObjectDaoImpl implements DomainObjectDao
{
    /**
     * Provides lookup methods for loading UML data types
     */
    private UmlTypeDao umlTypeDao;

    /**
     * Loads the domain objects contained in the specified input stream
     *
     * @param in            An input stream to the artifact that contains the domain objects
     *
     * @return              A List of domain objects contained in the input stream
     */
    @Override
    public DomainModel loadDomainModel( InputStream in )
    {
        try
        {
            DomainModel domainModel = new DomainModel();

            // Load the XMI document
            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build( in );
            Element root = doc.getRootElement();

            String version = root.getAttributeValue( "xmi.version" );
            System.out.println( "Version=" + version );

            Namespace ns = root.getNamespace();
            Namespace umlns = root.getNamespace( "UML" );
            Element content = root.getChild( "XMI.content", ns );
            Element model = content.getChild( "Model", umlns );
            domainModel.setName( model.getAttributeValue( "model" ) );
            System.out.println( "Model Name: " + model.getAttributeValue( "name" ) );

            // Load the packages in the model
            Map<String, String> classIdNameMap = new HashMap<String, String>();
            Map<String, Stereotype> classStereotypeMap = new HashMap<String, Stereotype>();
            List<Element> packageElements = model.getChild( "Namespace.ownedElement", umlns ).getChildren( "Package", umlns );

            //
            // Pre-process the class ids and names so that we can map types in the model to their correct object types
            //
            for( Element packageElement : packageElements )
            {
                // Save the package name
                String packageName = packageElement.getAttributeValue( "name" );
                domainModel.getPackages().add( packageName );

                // Load the classes in this package and store them in the classIdMap for type resolution
                Element ownedElement = packageElement.getChild( "Namespace.ownedElement", umlns );
                //List<Element> classList = packageElement.getChild( "Namespace.ownedElement", umlns ).getChildren( "Class", umlns );
                List<Element> classList = ownedElement.getChildren( "Class", umlns );
                for( Element classElement : classList )
                {
                    classIdNameMap.put( classElement.getAttributeValue( "xmi.id" ), packageName + "." + classElement.getAttributeValue( "name" ) );
                }

                // Load all of the class stereotypes in this package
                List<Element> stereotypeList = ownedElement.getChildren( "Stereotype", umlns );
                for( Element stereotypeElement : stereotypeList )
                {
                    // Create a Stereotype
                    Stereotype classStereotype = new Stereotype( packageName, stereotypeElement.getAttributeValue( "name" ), Stereotype.Type.CLASS );

                    // Add it to the domain model
                    domainModel.addStereotype( classStereotype );

                    // Put it in our lookup map
                    classStereotypeMap.put( stereotypeElement.getAttributeValue( "xmi.id" ), classStereotype );
                }

                // TODO: debug, remove
                System.out.println( "Class Stereotypes:" );
                for( String id : classStereotypeMap.keySet() )
                {
                    System.out.println( "\t" + classStereotypeMap.get( id ) );
                }
            }

            //
            // Load all non-class stereotypes in the model
            //
            Map<String, Stereotype> stereotypeMap = new HashMap<String, Stereotype>();
            List<Element> stereotypeElements = model.getChild( "Namespace.ownedElement", umlns ).getChildren( "Stereotype", umlns );
            for( Element stereotypeElement : stereotypeElements )
            {
                String stereotypeId = stereotypeElement.getAttributeValue( "xmi.id" );
                String stereotypeName = stereotypeElement.getAttributeValue( "name" );

                // Load the type
                String baseClass = stereotypeElement.getChildTextTrim( "Stereotype.baseClass", umlns );
                Stereotype.Type stereotypeType = null;
                if( baseClass.equalsIgnoreCase( "Attribute" ) )
                {
                    stereotypeType = Stereotype.Type.ATTRIBUTE;
                }
                else if( baseClass.equalsIgnoreCase( "AssociationEnd" ) )
                {
                    stereotypeType = Stereotype.Type.ASSOCIATION_END;
                }
                else
                {
                    System.out.println( "Unknown stereotype: " + stereotypeName + " = " + baseClass );
                }

                // Create the stereotype
                Stereotype stereotype = new Stereotype( stereotypeName, stereotypeType );

                // Add this stereotype to the domain model
                domainModel.addStereotype( stereotype );
                //domainModel.getStereotypes().add( stereotype );

                // Add this stereotype to our lookup map
                stereotypeMap.put( stereotypeId, stereotype );
            }

            // TODO: debug, remove
            System.out.println( "Other Stereotypes:" );
            for( String id : stereotypeMap.keySet() )
            {
                System.out.println( "\t" + stereotypeMap.get( id ) );
            }

            //
            // Process the classes and build our list of domain objects
            //
            Map<String, DomainObject> classIdDomainObjectMap = new HashMap<String, DomainObject>();
            List<DomainObject> domainObjects = new ArrayList<DomainObject>();
            for( Element packageElement : packageElements )
            {
                // Load the package name, which we'll associate with our domain objects
                String packageName = packageElement.getAttributeValue( "name" );
                System.out.println( "Package name: " + packageName );

                // Load the classes in this package
                Element ownedElement = packageElement.getChild( "Namespace.ownedElement", umlns );
                List<Element> classList = ownedElement.getChildren( "Class", umlns );
                for( Element classElement : classList )
                {
                    // Create a domain object
                    DomainObject domainObject = new DomainObject();
                    domainObject.setName( classElement.getAttributeValue( "name" ) );
                    domainObject.setPackageName( packageName );
                    System.out.println( "Class name: " + domainObject.getName() );

                    // Load class stereotypes
                    Element classStereotypeElement = classElement.getChild( "ModelElement.stereotype", umlns );
                    if( classStereotypeElement != null )
                    {
                        List<Element> classStereotypeElementList = classStereotypeElement.getChildren( "Stereotype", umlns );
                        for( Element stereotypeElement : classStereotypeElementList )
                        {
                            domainObject.getStereotypes().add( classStereotypeMap.get(  stereotypeElement.getAttributeValue( "xmi.idref" ) ) );
                        }
                    }

                    // Save the class id and domain object so that we can add relationships
                    classIdDomainObjectMap.put( classElement.getAttributeValue( "xmi.id" ), domainObject );

                    // Load the class's attributes
                    List<Element> attributeList = classElement.getChild( "Classifier.feature", umlns ).getChildren( "Attribute", umlns );
                    for( Element attributeElement : attributeList )
                    {
                        String attributeName = attributeElement.getAttributeValue( "name" );
                        System.out.print( "\t" + attributeName );

                        // Build this field
                        Field field = new Field( attributeName );

                        // Load the multiplicity (if necessary)

                        // Load the type
                        Element typeParentElement = attributeElement.getChild( "StructuralFeature.type", umlns );
                        List<Element> typeNodes = typeParentElement.getChildren();
                        for( Element typeElement : typeNodes )
                        {
                            // TODO: primary key is current hard coded to "id", need to create a stereotype <<pk>> and use it
                            if( typeElement.getName().equals( "DataType" ) )
                            {
                                String href = typeElement.getAttributeValue( "href" );
                                System.out.println( "\t\t" + umlTypeDao.getRemoteType( href ) );

                                // Set the field's type
                                field.setType( umlTypeDao.getRemoteType( href ) );

                                //boolean primaryKey = attributeName.equalsIgnoreCase( "id" );
                                //domainObject.getFields().add( new Field( attributeName, umlTypeDao.getRemoteType( href ), primaryKey ) );
                            }
                            else if( typeElement.getName().equals( "Class" ) )
                            {
                                String href = typeElement.getAttributeValue( "href" );
                                if( href != null )
                                {
                                    //System.out.println( "\t\t" + umlTypeDao.getRemoteType( href ) );
                                    //boolean primaryKey = attributeName.equalsIgnoreCase( "id" );

                                    // Set the field's type
                                    field.setType( umlTypeDao.getRemoteType( href ) );
                                    //domainObject.getFields().add( new Field( attributeName, umlTypeDao.getRemoteType( href ), primaryKey ) );
                                }
                                else
                                {
                                    href = typeElement.getAttributeValue( "xmi.idref" );
                                    if( href != null )
                                    {
                                        if( classIdNameMap.containsKey( href ) )
                                        {
                                            System.out.println( "\t\t" + classIdNameMap.get( href ) );

                                            // Set the field's type
                                            field.setType( classIdNameMap.get( href ) );

                                            //boolean primaryKey = attributeName.equalsIgnoreCase( "id" );
                                            //domainObject.getFields().add( new Field( attributeName, classIdNameMap.get( href ), primaryKey ) );
                                        }
                                        else
                                        {
                                            System.out.println( "Unknown type (xmi.idref): " + href );
                                        }
                                    }
                                    else
                                    {
                                        System.out.println( "\t\thref is null" );
                                    }
                                }
                            }
                            else
                            {
                                System.out.println( "Unknown type: " + typeElement.getName() );
                            }
                        }

                        // Load the stereotypes implemented by this attribute
                        Element attributeStereotypeElements = attributeElement.getChild( "ModelElement.stereotype", umlns );
                        if( attributeStereotypeElements != null )
                        {
                            List<Element> attributeStereotypeElementList = attributeStereotypeElements.getChildren( "Stereotype", umlns );
                            for( Element attributeStereotypeElement : attributeStereotypeElementList )
                            {
                                // Load the stereotype from the map
                                Stereotype attributeStereotype = stereotypeMap.get( attributeStereotypeElement.getAttributeValue( "xmi.idref" ) );

                                // Add it to the list of this field's stereotypes
                                field.getStereotypes().add( attributeStereotype );

                                // Check for the "pk" stereotype, which indicates that this is the primary key for the object
                                if( attributeStereotype.getName().equalsIgnoreCase( "pk" ) )
                                {
                                    field.setPrimaryKey( true );
                                }
                            }
                        }


                        // Add the field to the domain object
                        domainObject.getFields().add( field );
                    }

                    // Load the class's operations
                    List<Element> operationList = classElement.getChild( "Classifier.feature", umlns ).getChildren( "Operation", umlns );
                    for( Element operationElement : operationList )
                    {
                        // Build the new operation
                        Operation operation = new Operation( operationElement.getAttributeValue( "name" ) );

                        // Set its visibility
                        String visibility = operationElement.getAttributeValue( "visibility" );
                        if( visibility.equalsIgnoreCase( "public" ) )
                        {
                            operation.setVisibility( Operation.Visibility.PUBLIC );
                        }
                        else if( visibility.equalsIgnoreCase( "protected" ) )
                        {
                            operation.setVisibility( Operation.Visibility.PROTECTED );
                        }
                        else if( visibility.equalsIgnoreCase( "private" ) )
                        {
                            operation.setVisibility( Operation.Visibility.PRIVATE );
                        }

                        // Find its return value and parameters
                        List<Element> parameterList = operationElement.getChild( "BehavioralFeature.parameter", umlns ).getChildren( "Parameter", umlns );
                        for( Element parameterElement : parameterList )
                        {
                            // Create the operation parameter
                            OperationParameter operationParameter = new OperationParameter( parameterElement.getAttributeValue( "name" ) );
                            List<Element> parameterTypeElements = parameterElement.getChild( "Parameter.type", umlns ).getChildren();
                            if( parameterTypeElements.size() > 0 )
                            {
                                System.out.println( "Found " + parameterTypeElements.size() + " types for parameter " + operationParameter.getName() + " for operation " + operation.getName() );
                            }

                            // Load the operation type
                            Element typeElement = parameterTypeElements.get( 0 );
                            if( typeElement.getName().equals( "DataType" ) )
                            {
                                //  The operation parameter is a simple type
                                String href = typeElement.getAttributeValue( "href" );
                                operationParameter.setType( umlTypeDao.getRemoteType( href ) );
                            }
                            else if( typeElement.getName().equals( "Class" ) )
                            {
                                String href = typeElement.getAttributeValue( "href" );
                                if( href != null )
                                {
                                    // The operation parameter is a remote class type (global type)
                                    operationParameter.setType( umlTypeDao.getRemoteType( href ) );
                                }
                                else
                                {
                                    href = typeElement.getAttributeValue( "xmi.idref" );
                                    if( href != null )
                                    {
                                        if( classIdNameMap.containsKey( href ) )
                                        {
                                            // The operation is a local type (one of our our own classes)
                                            operationParameter.setType( classIdNameMap.get( href ) );
                                        }
                                        else
                                        {
                                            System.out.println( "Unknown type (xmi.idref): " + href );
                                        }
                                    }
                                    else
                                    {
                                        System.out.println( "\t\thref is null" );
                                    }
                                }
                            }
                            else if( typeElement.getName().equals( "Interface" ) )
                            {
                                String href = typeElement.getAttributeValue( "href" );
                                if( href != null )
                                {
                                    // The operation parameter is a remote class type (global type)
                                    operationParameter.setType( umlTypeDao.getRemoteType( href ) );
                                }
                                else
                                {
                                    href = typeElement.getAttributeValue( "xmi.idref" );
                                    if( href != null )
                                    {
                                        if( classIdNameMap.containsKey( href ) )
                                        {
                                            // The operation is a local type (one of our our own classes)
                                            operationParameter.setType( classIdNameMap.get( href ) );
                                        }
                                        else
                                        {
                                            System.out.println( "Unknown type (xmi.idref): " + href );
                                        }
                                    }
                                    else
                                    {
                                        System.out.println( "\t\thref is null" );
                                    }
                                }
                            }
                            else
                            {
                                System.out.println( "Unknown type: " + typeElement.getName() );
                            }

                            // Add the operation parameter to the operation
                            if( operationParameter.getName().equalsIgnoreCase( "return" ) )
                            {
                                // This is the return type
                                operation.setReturnType( operationParameter.getType() );
                            }
                            else
                            {
                                operation.getParameters().add( operationParameter );
                            }
                            // MONKEY

                        }

                        // Add the operation to the domain object
                        domainObject.getOperations().add( operation );
                    }

                    // Add the domain object to our list
                    domainModel.getDomainObjects().add( domainObject );
                    //domainObjects.add( domainObject );
                }

                // Load the package-level associations (relationships)
                List<Element> associationList = ownedElement.getChildren( "Association", umlns );
                for( Element associationElement : associationList )
                {
                    String associationName = associationElement.getAttributeValue( "name" );
                    System.out.println( "Association Name: " + associationName );

                    // Load the two ends of the association
                    List<Element> associationEnds = associationElement.getChild( "Association.connection", umlns ).getChildren( "AssociationEnd", umlns );
                    if( associationEnds.size() == 2 )
                    {
                        // Load the "left" side of the relationship
                        Element associationLeft = associationEnds.get( 0 );
                        String leftName = associationLeft.getAttributeValue( "name" );
                        String leftClassId = associationLeft.getChild( "AssociationEnd.participant", umlns ).getChild( "Class", umlns ).getAttributeValue( "xmi.idref" );
                        DomainObject leftDomainObject = classIdDomainObjectMap.get( leftClassId );
                        Element leftMultiplicity = associationLeft.getChild( "AssociationEnd.multiplicity", umlns ).getChild( "Multiplicity", umlns ).getChild( "Multiplicity.range", umlns ).getChild( "MultiplicityRange", umlns );
                        int leftLower = Integer.parseInt( leftMultiplicity.getAttributeValue( "lower" ) );
                        int leftUpper = Integer.parseInt( leftMultiplicity.getAttributeValue( "upper" ) );
                        Relationship.LoadStrategy leftLoadStrategy = Relationship.LoadStrategy.LAZY;
                        Element leftStereotypes = associationLeft.getChild( "ModelElement.stereotype", umlns );
                        if( leftStereotypes != null )
                        {
                            List<Element> associationStereotypeList = leftStereotypes.getChildren( "Stereotype", umlns );
                            for( Element associationStereotypeElement : associationStereotypeList )
                            {
                                Stereotype stereotype = stereotypeMap.get( associationStereotypeElement.getAttributeValue( "xmi.idref" ) );
                                if( stereotype != null && stereotype.getName().equalsIgnoreCase( "eager" ) )
                                {
                                    leftLoadStrategy = Relationship.LoadStrategy.EAGER;
                                }
                            }
                        }

                        // Load the "right" side of the relationship
                        Element associationRight = associationEnds.get( 1 );
                        String rightName = associationRight.getAttributeValue( "name" );
                        String rightClassId = associationRight.getChild( "AssociationEnd.participant", umlns ).getChild( "Class", umlns ).getAttributeValue( "xmi.idref" );
                        DomainObject rightDomainObject = classIdDomainObjectMap.get( rightClassId );
                        Element rightMultiplicity = associationRight.getChild( "AssociationEnd.multiplicity", umlns ).getChild( "Multiplicity", umlns ).getChild( "Multiplicity.range", umlns ).getChild( "MultiplicityRange", umlns );
                        int rightLower = Integer.parseInt( rightMultiplicity.getAttributeValue( "lower" ) );
                        int rightUpper = Integer.parseInt( rightMultiplicity.getAttributeValue( "upper" ) );
                        Relationship.LoadStrategy rightLoadStrategy = Relationship.LoadStrategy.LAZY;
                        Element rightStereotypes = associationRight.getChild( "ModelElement.stereotype", umlns );
                        if( rightStereotypes != null )
                        {
                            List<Element> associationStereotypeList = rightStereotypes.getChildren( "Stereotype", umlns );
                            for( Element associationStereotypeElement : associationStereotypeList )
                            {
                                Stereotype stereotype = stereotypeMap.get( associationStereotypeElement.getAttributeValue( "xmi.idref" ) );
                                if( stereotype != null && stereotype.getName().equalsIgnoreCase( "eager" ) )
                                {
                                    rightLoadStrategy = Relationship.LoadStrategy.EAGER;
                                }
                            }
                        }

                        System.out.println( "Relationship: "
                                            + leftName + ": "
                                            + leftDomainObject.getName()
                                            + "(" + leftLower + ", " + leftUpper + "), "
                                            + leftLoadStrategy + " "
                                            + " -> "
                                            + rightName + ": "
                                            + rightDomainObject.getName()
                                            + "(" + rightLower + ", " + rightUpper + "), "
                                            + rightLoadStrategy );


                        // Derive the relationship type and add it to the two domain objects
                        // TODO: need to add the load strategy (lazy vs eager) to the model and read it somehow
                        if( leftUpper == 1 && rightUpper == 1 )
                        {
                            leftDomainObject.getRelationships().add( new Relationship( rightName, leftName, rightDomainObject, Relationship.Type.ONE_TO_ONE, rightLoadStrategy, true ) );
                            rightDomainObject.getRelationships().add( new Relationship( leftName, rightName, leftDomainObject, Relationship.Type.ONE_TO_ONE, leftLoadStrategy, false ) );
                        }
                        else if( leftUpper == 1 && rightUpper == -1 )
                        {
                            leftDomainObject.getRelationships().add( new Relationship( rightName, leftName, rightDomainObject, Relationship.Type.ONE_TO_MANY, rightLoadStrategy ) );
                            rightDomainObject.getRelationships().add( new Relationship( leftName, rightName, leftDomainObject, Relationship.Type.MANY_TO_ONE, leftLoadStrategy ) );
                        }
                        else if( leftUpper == -1 && rightUpper == 1 )
                        {
                            leftDomainObject.getRelationships().add( new Relationship( rightName, leftName, rightDomainObject, Relationship.Type.MANY_TO_ONE, rightLoadStrategy ) );
                            rightDomainObject.getRelationships().add( new Relationship( leftName, rightName, leftDomainObject, Relationship.Type.ONE_TO_MANY, leftLoadStrategy ) );
                        }
                        else if( leftUpper == -1 && rightUpper == -1 )
                        {
                            leftDomainObject.getRelationships().add( new Relationship( rightName, leftName, rightDomainObject, Relationship.Type.MANY_TO_MANY, rightLoadStrategy, true ) );
                            rightDomainObject.getRelationships().add( new Relationship( leftName, rightName, leftDomainObject, Relationship.Type.MANY_TO_MANY, leftLoadStrategy, false ) );
                        }
                    }
                    else
                    {
                        System.out.println( "Association Ends: " + associationEnds.size() );
                    }
                }
            }

            // TODO: remove, debug
            for( DomainObject domainObject : domainModel.getDomainObjects() )
            {
                System.out.println( "Class: " + domainObject.getPackageName() + "." + domainObject.getName() );
                for( Field field : domainObject.getFields() )
                {
                    System.out.println( "\t" + field.getName() + ": " + field.getType() );
                }
            }

            // Return our domain objects
            //return domainObjects;
            return domainModel;

        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Returns a reference to the UmlTypeDao implementation class
     *
     * @return              A reference to the UmlTypeDao implementation class
     */
    public UmlTypeDao getUmlTypeDao()
    {
        return umlTypeDao;
    }

    /**
     * Sets the UmlTypeDao implementation class
     *
     * @param umlTypeDao    The UmlTypeDao implementation class
     */
    public void setUmlTypeDao( UmlTypeDao umlTypeDao )
    {
        this.umlTypeDao = umlTypeDao;
    }
}
