package com.geekcap.modeler.maven;

import com.geekcap.modeler.dao.PropertiesFileUmlTypeDaoImpl;
import com.geekcap.modeler.dao.Xmi14DomainObjectDaoImpl;
import com.geekcap.modeler.service.ModelerService;
import com.geekcap.modeler.service.ModelerServiceImpl;
import com.geekcap.modeler.service.TemplateService;
import com.geekcap.modeler.service.VelocityTemplateServiceImpl;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;

/**
 * Modeler Maven 2 Plugin
 *
 * @author shaines
 *
 * @goal execute
 * @phase generate-sources
 */
public class ModelerMojo extends AbstractMojo
{
    /**
     * The directory that contains your project's XMI document(s)
     * 
     * @parameter alias="xmidir" expression="${modeler.xmldir}" default-value="src/main/uml"
     */
    private String xmiDir;

    /**
     * @parameter expression="${project}"
     * @required
     * @readonly
     * @since 1.0
     */
    private MavenProject project;

    private ModelerService modelerService;

    public ModelerMojo()
    {
        // Wire together a ModelerService
        TemplateService templateService = new VelocityTemplateServiceImpl();
        Xmi14DomainObjectDaoImpl dao = new Xmi14DomainObjectDaoImpl();
        dao.setUmlTypeDao( new PropertiesFileUmlTypeDaoImpl() );
        modelerService = new ModelerServiceImpl( templateService, dao );
    }

    /**
     * Main execution of the modeler plugin
     *
     * @throws MojoExecutionException
     * @throws MojoFailureException
     */
    @Override
    public void execute() throws MojoExecutionException, MojoFailureException
    {
        // Log what we're doing
        getLog().info( "xmidir: " + xmiDir );

        if( xmiDir == null )
        {
            throw new MojoExecutionException( "Cannot execute if the XMI directory is null" );
        }

        try
        {
            // Find the XMI directory
            File dir = new File( xmiDir );
            if(  ! dir.isDirectory() )
            {
                throw new MojoExecutionException( "XMI Directory is not a directory: " + xmiDir );
            }

            // Build a list of XMI files in the directory
            List<String> xmiFilenames = new ArrayList<String>();
            File[] files = dir.listFiles();
            for( File file : files )
            {
                getLog().debug( "Examining file: " + file.getAbsolutePath() );
                if( file.getAbsolutePath().toLowerCase().endsWith( ".xmi" ) )
                {
                    xmiFilenames.add( file.getAbsolutePath() );
                }
            }

            // Process the XMI files
            String destinationPath = "target" + File.separator + "generated-sources" + File.separator + "uml";
            for( String xmiFilename : xmiFilenames )
            {
                getLog().info( "Processing XMI file: " + xmiFilename );
                try
                {
                    modelerService.generate( new FileInputStream( xmiFilename ), destinationPath, "artifact" );
                }
                catch( Exception e )
                {
                    throw new MojoExecutionException( "Modeler execution failed when generating artifacts for XMI file: " + xmiFilename + ": " + e.getMessage(), e );
                }
            }

        }
        catch( Exception e )
        {
            throw new MojoExecutionException( "Modeler execution failed: " + e.getMessage(), e );
        }

        // Add our generated source directory to the project's list of compile directories
        project.addCompileSourceRoot( "target/generated-sources/uml" );

        //throw new UnsupportedOperationException( "Not supported yet." );
    }
}
