/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

/**
 * Represents an entire domain model.
 * 
 * @author shaines
 */
public class DomainModel
{
    /**
     * The name of the DomainObject
     */
    private String name;

    /**
     * A List of all DomainObjects in the model
     */
    private List<DomainObject> domainObjects = new ArrayList<DomainObject>();

    /**
     * A set that contains all class-level stereotypes in the model
     */
    private Set<Stereotype> stereotypes = new TreeSet<Stereotype>();

    /**
     * A map of stereotype types to a list of the stereotypes that implement those stereotypes
     */
    private Map<Stereotype.Type,Set<Stereotype>> stereotypeMap = new HashMap<Stereotype.Type,Set<Stereotype>>();

    /**
     * A set that contains the names of all packages in the model
     */
    private Set<String> packages = new TreeSet<String>();

    public DomainModel()
    {
    }

    public DomainModel( String name )
    {
        this.name = name;
    }

    public String getName()
    {
        return name;
    }

    public void setName( String name )
    {
        this.name = name;
    }

    public List<DomainObject> getDomainObjects()
    {
        return domainObjects;
    }

    /**
     * Returns all DomainObjects that implement the specified stereotype
     *
     * @param name          The name of the stereotype (package name excluded) for which
     *                      to find domain objects
     *
     * @return              A list of all objects that implement the requested stereotype
     */
    public List<DomainObject> findDomainObjectsByStereotype( String name )
    {
        List<DomainObject> dos = new ArrayList<DomainObject>();
        for( DomainObject domainObject : domainObjects )
        {
            if( domainObject.containsStereotype( name ) )
            {
                dos.add( domainObject );
            }
        }
        return dos;
    }

    /**
     * Returns a set of all package names defined in the domain model
     *
     * @return              All package names defined in the domain model
     */
    public Set<String> getPackages()
    {
        return packages;
    }

    /**
     * Adds the specified stereotype to the domain model
     *
     * @param stereotype    The stereotype to add to the domain model
     */
    public void addStereotype( Stereotype stereotype )
    {
        Set<Stereotype> stereotypeSet = null;
        if( stereotypeMap.containsKey( stereotype.getType() ) )
        {
            stereotypeSet = stereotypeMap.get( stereotype.getType() );
        }
        else
        {
            stereotypeSet = new TreeSet<Stereotype>();
            stereotypeMap.put( stereotype.getType(), stereotypeSet );
        }
        stereotypeSet.add( stereotype );
    }

    /**
     * Returns the types of stereotypes defined in this domain model
     *
     * @return              A Set of Stereotype.Type objects for all stereotypes defined in this domain model
     */
    public Set<Stereotype.Type> getStereotypeTypes()
    {
        return stereotypeMap.keySet();
    }

    /**
     * Returns all stereotypes of the specified type
     *
     * @param type          The type of stereotypes to return
     *
     * @return              A Set of all Stereotypes defined in the domain model of the requested type
     */
    public Set<Stereotype> getStereotypes( Stereotype.Type type )
    {
        return stereotypeMap.get( type );
    }
}

