/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a domain object in a model
 * 
 * @author shaines
 */
public class DomainObject implements Serializable
{
    /**
     * The name of the domain object
     */
    private String name;

    /**
     * The name of the package that contains this domain object
     */
    private String packageName;

    /**
     * A list of all of the fields in this domain object
     */
    private List<Field> fields = new ArrayList<Field>();

    /**
     * A list of all of the relationships (to other objects) for this domain object
     */
    private List<Relationship> relationships = new ArrayList<Relationship>();

    /**
     * A list of all stereotypes implemented by this domain object
     */
    private List<Stereotype> stereotypes = new ArrayList<Stereotype>();

    /**
     * A list of all operations defined for this domain object
     */
    private List<Operation> operations = new ArrayList<Operation>();

    /**
     * Creates a new DomainObject
     */
    public DomainObject()
    {
    }

    /**
     * Creates a new DomainObject with the specified name
     *
     * @param name          The name of the domain object
     */
    public DomainObject( String name )
    {
        this.name = name;
    }

    /**
     * Returns the name of the domain object
     *
     * @return              The name of the domain object
     */
    public String getName()
    {
        return name;
    }

    /**
     * Sets the name of the domain object
     *
     * @param name          The name of the domain object
     */
    public void setName( String name )
    {
        this.name = name;
    }

    /**
     * Returns the name of the package that contains this domain object
     *
     * @return              The name of the package that contains this domain object
     */
    public String getPackageName()
    {
        return packageName;
    }

    /**
     * Sets the name of the package that contains this domain object
     *
     * @param packageName   The name of the package that contains this domain object
     */
    public void setPackageName( String packageName )
    {
        this.packageName = packageName;
    }

    /**
     * Returns the primary key of this domain object; note that the current implementation
     * does not support domain objects with multiple primary keys
     *
     * @return              A Field containing the primary key of this domain object
     */
    public Field getPrimaryKey()
    {
        for( Field field : fields )
        {
            if( field.isPrimaryKey() )
            {
                return field;
            }
        }
        return null;
    }

    /**
     * Returns the list of fields for this domain object. The field list is already instantiated, so
     * to add fields to the domain object, invoke do.getFields().add( new Field( ... ) )
     *
     * @return              The List of fields for this domain object
     */
    public List<Field> getFields()
    {
        return fields;
    }

    /**
     * Returns the list of relationships for this domain object. The relationships list is already
     * instantiated, so to add relationships to the domain object, invoke:
     *  do.getRelationships().add( new Relationship( .. ) );
     *
     * @return              The list of relationships for this domain object
     */
    public List<Relationship> getRelationships()
    {
        return relationships;
    }

    /**
     * Returns the list of stereotypes that this domain object implements
     *
     * @return              The list of stereotypes that this domain model implements
     */
    public List<Stereotype> getStereotypes()
    {
        return stereotypes;
    }

    /**
     * Return the list of operations defined for this domain object
     *
     * @return              The list of operations defined for this domain object
     */
    public List<Operation> getOperations()
    {
        return operations;
    }

    /**
     * Returns true if one or more of the relationships that this domain object
     * maintains is one-to-one
     *
     * @return              True if one or more of this domain object's relationships is one-to-one
     */
    public boolean hasOneToOneRelationship()
    {
        for( Relationship relationship : relationships )
        {
            if( relationship.isOneToOne() )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true if one or more of the relationships that this domain object
     * maintains is one-to-many
     *
     * @return              True if one or more of this domain object's relationships is one-to-many
     */
    public boolean hasOneToManyRelationship()
    {
        for( Relationship relationship : relationships )
        {
            if( relationship.isOneToMany() )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true if one or more of the relationships that this domain object
     * maintains is many-to-one
     *
     * @return              True if one or more of this domain object's relationships is many-to-one
     */
    public boolean hasManyToOneRelationship()
    {
        for( Relationship relationship : relationships )
        {
            if( relationship.isManyToOne() )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true if one or more of the relationships that this domain object
     * maintains is many-to-many
     *
     * @return              True if one or more of this domain object's relationships is many-to-many
     */
    public boolean hasManyToManyRelationship()
    {
        for( Relationship relationship : relationships )
        {
            if( relationship.isManyToMany() )
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true if the DomainObject contains the named stereotype
     *
     * @param name              The name of the stereotype to check for
     *
     * @return                  True if the domain object contains the stereotype, else false
     */
    public boolean containsStereotype( String name )
    {
        for( Stereotype stereotype : stereotypes )
        {
            if( stereotype.getName().equalsIgnoreCase( name ) )
            {
                return true;
            }
        }
        return false;
    }
}
