/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.model;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a domain object relationship, e.g. one-to-one, one-to-many, etc. It maintains
 * the relationship type, the target of the relationship, and the load strategy (lazy or eager),
 * which may be used by templates for defining DAO functionality.
 *
 * @author shaines
 */
public class Relationship
{
    /**
     * Defines types of relationships that DomainObjects can have between each other
     */
    public enum Type
    {
        ONE_TO_ONE, ONE_TO_MANY, MANY_TO_ONE, MANY_TO_MANY

    }

    /**
     * The load strategy defines how the persistence engine should load objects in this
     * relationship: lazy = load on demand, eager = load at object creation type
     */
    public enum LoadStrategy
    {
        LAZY, EAGER

    }
    /**
     * The name of the relationship, which, if present, will be reflected in the name of
     * the related field in generated code
     */
    private String name;

    /**
     * The name of the relationship on the other side of the association
     */
    private String inverseName;

    /**
     * The type of relationship
     */
    private Type type;

    /**
     * The LoadStrategy
     */
    private LoadStrategy loadStrategy;

    /**
     * The target of the relationship
     */
    private DomainObject target;

    /**
     * Some relationships require that one object own the relationship (e.g. one-to-one),
     * so this boolean defines the ownership status of an object for those relationships
     */
    private boolean owner;

    private List<Stereotype> stereotypes = new ArrayList<Stereotype>();

    /**
     * Creates a new Relationship
     */
    public Relationship()
    {
    }

    /**
     * Creates a new Relationship
     *
     * @param name              The name of the relationship
     * @param invereName        The name of the relationship on the other side of the association
     * @param target            The target of the relationship: the DomainObject on the other side of this relationship
     * @param type              The type of the relationship (ONE_TO_ONE, ONE_TO_MANY, etc.)
     * @param loadStrategy      The load strategy (LAZY or EAGER)
     */
    public Relationship( String name, String inverseName, DomainObject target, Type type, LoadStrategy loadStrategy )
    {
        this.name = name;
        this.inverseName = inverseName;
        this.target = target;
        this.type = type;
        this.loadStrategy = loadStrategy;
    }

    /**
     * Creates a new Relationship
     *
     * @param name              The name of the relationship
     * @param invereName        The name of the relationship on the other side of the association
     * @param target            The target of the relationship: the DomainObject on the other side of this relationship
     * @param type              The type of the relationship (ONE_TO_ONE, ONE_TO_MANY, etc.)
     * @param loadStrategy      The load strategy (LAZY or EAGER)
     * @param owner             True if this is the owning side of the relationship, false if it is not
     */
    public Relationship( String name, String inverseName, DomainObject target, Type type, LoadStrategy loadStrategy, boolean owner )
    {
        this.name = name;
        this.inverseName = inverseName;
        this.target = target;
        this.type = type;
        this.loadStrategy = loadStrategy;
        this.owner = owner;
    }

    /**
     * Creates a new Relationship with a default load strategy of LAZY
     *
     * @param name              The name of the relationship
     * @param invereName        The name of the relationship on the other side of the association
     * @param target            The target of the relationship: the DomainObject on the other side of this relationship
     * @param type              The type of the relationship (ONE_TO_ONE, ONE_TO_MANY, etc.)
     */
    public Relationship( String name, String inverseName, DomainObject target, Type type )
    {
        this.name = name;
        this.inverseName = inverseName;
        this.target = target;
        this.type = type;
        this.loadStrategy = LoadStrategy.LAZY;
    }

    /**
     * Returns the load strategy for this relationship
     *
     * @return                  The load strategy for this relationship
     */
    public LoadStrategy getLoadStrategy()
    {
        return loadStrategy;
    }

    /**
     * Sets the load strategy for this relationship
     *
     * @param loadStrategy      The load strategy for this relationship
     */
    public void setLoadStrategy( LoadStrategy loadStrategy )
    {
        this.loadStrategy = loadStrategy;
    }

    /**
     * Returns the target domain object for this relationship
     *
     * @return                  The target domain object for this relationship
     */
    public DomainObject getTarget()
    {
        return target;
    }

    /**
     * Sets the target domain object for this relationship
     *
     * @param target            The target domain object for this relationship
     */
    public void setTarget( DomainObject target )
    {
        this.target = target;
    }

    /**
     * Returns the name of the relationship
     * 
     * @return                  The name of the relationship
     */
    public String getName()
    {
        return name;
    }

    /**
     * Sets the name of the relationship
     *
     * @param name              The name of the relationship
     */
    public void setName( String name )
    {
        this.name = name;
    }

    /**
     * Returns the name of the relationship on the other side of the association
     *
     * @return                  The name of the relationship on the other side of the association
     */
    public String getInverseName()
    {
        return inverseName;
    }

    /**
     * Sets the name of the relationship on the other side of the association
     *
     * @param inverseName       The name of the relationship on the other side of the association
     */
    public void setInverseName( String inverseName )
    {
        this.inverseName = inverseName;
    }



    /**
     * Returns the type of the relationship
     *
     * @return                  The type of the relationship (ONE_TO_ONE, ONE_TO_MANY, etc.)
     */
    public Type getType()
    {
        return type;
    }

    /**
     * Sets the type of the relationship
     *
     * @param type              The type of the relationship (ONE_TO_ONE, ONE_TO_MANY, etc.)
     */
    public void setType( Type type )
    {
        this.type = type;
    }

    /**
     * Returns true if this is the owning side of the relationship, false if it is not
     *
     * @return                  True if this is the owning side of the relationship, false if it is not
     */
    public boolean isOwner()
    {
        return owner;
    }

    /**
     * Sets the ownership status of this relationship: true if this is the owning side of the relationship,
     * false if it is not.
     *
     * @param owner             True if this is the owning side of the relationship, false if it is not
     */
    public void setOwner( boolean owner )
    {
        this.owner = owner;
    }

    /**
     * Returns the list of stereotypes for this relationship
     *
     * @return                  The list of stereotypes implemented by this relationship
     */
    public List<Stereotype> getStereotypes()
    {
        return stereotypes;
    }

    public boolean isOneToOne()
    {
        return type == Type.ONE_TO_ONE;
    }

    public boolean isManyToMany()
    {
        return type == Type.MANY_TO_MANY;
    }

    public boolean isManyToOne()
    {
        return type == Type.MANY_TO_ONE;
    }

    public boolean isOneToMany()
    {
        return type == Type.ONE_TO_MANY;
    }
}
