/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.util;

import com.geekcap.modeler.model.DomainObject;
import com.geekcap.modeler.model.Field;

/**
 * Defines conversion utils for things like table/column names to Java names, etc.
 *
 * @author shaines
 */
public class ConversionUtil
{
    /**
     * Converts a domain object's name to a table name following these rules:
     *      - The name is converted to all upper case
     *      - All camel-case changes are prefixed by an underscore (first character is ignored)
     * 
     * For example: TableName would be converted to TABLE_NAME
     * 
     * @param domainObject          The domain object to convert
     * 
     * @return                      A table name for the domain object
     */
    public static String getTableName( DomainObject domainObject )
    {
        return getDatabaseName( domainObject.getName() );
    }

    /**
     * Converts a field's name to a column name following these rules:
     *      - The name is converted to all upper case
     *      - All camel-case changes are prefixed by an underscore (first character is ignored)
     *
     * For example: columnName would be converted to COLUMN_NAME
     *
     * @param domainObject          The domain object to convert
     *
     * @return                      A column name for the domain object
     */
    public static String getColumnName( Field field )
    {
        return getDatabaseName( field.getName() );
    }

    /**
     * Derives the foreign key name for the specified domain object. For example, if the object name is Customer
     * and the primary key name is id then the table name is CUSTOMER and the primary key is ID, so the foreign
     * key name would be CUSTOMER_ID.
     *
     * @param domainObject          The object from which to derive the foreign key name
     *
     * @return                      The foreign key name
     */
    public static String getForeignKey( DomainObject domainObject )
    {
        return getDatabaseName( domainObject.getName() ) + "_" + getDatabaseName( domainObject.getPrimaryKey().getName() );
    }

        /**
     * Derives the foreign key name for the specified domain object. For example, if the object name is Customer
     * and the primary key name is id then the table name is CUSTOMER and the primary key is ID, so the foreign
     * key name would be CUSTOMER_ID.
     *
     * @param domainObject          The object from which to derive the foreign key name
     *
     * @return                      The foreign key name
     */
    public static String getPrimaryKey( DomainObject domainObject )
    {
        return getDatabaseName( domainObject.getPrimaryKey().getName() );
    }

    /**
     * Converts a field's name to a column name following these rules:
     *      - The name is converted to all upper case
     *      - All camel-case changes are prefixed by an underscore (first character is ignored)
     *
     * For example: columnName would be converted to COLUMN_NAME
     *
     * @param name                  The name object to convert
     *
     * @return                      A database name for the String
     */
    public static String getDatabaseName( String name )
    {
        StringBuilder sb = new StringBuilder();
        for( int i=0; i<name.length(); i++ )
        {
            if( i>0 && Character.isUpperCase( name.charAt( i ) ) )
            {
                sb.append( "_" );
            }
            sb.append( Character.toUpperCase( name.charAt( i ) ) );
        }

        return sb.toString();
    }

    /**
     * Converts a field name into a method name, e.g. name would return Name
     *
     * @param name                  The field name
     *
     * @return                      An appropriate method name
     */
    public static String getMethodName( String name )
    {
        return Character.toUpperCase( name.charAt( 0 ) ) + name.substring( 1 );
    }

    public static String getFieldName( String name )
    {
        return Character.toLowerCase( name.charAt( 0 ) ) + name.substring( 1 );
    }
}
