/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.geekcap.modeler.util;

import com.geekcap.modeler.model.DomainObject;
import com.geekcap.modeler.model.Operation;
import com.geekcap.modeler.model.OperationParameter;
import java.util.Set;
import java.util.TreeSet;

/**
 * Provides helper methods for working with repository domain objects
 *
 * @author shaines
 */
public class RepositoryUtils
{
    /**
     * Returns the fully qualified class name of the generic managed by this repository
     *
     * @param domainObject          The domain objec to examine
     *
     * @return                      The fully qualified class name of the generic managed
     *                              by this repository
     */
    public static String getFullyQualifiedGenericType( DomainObject domainObject )
    {
        String genericType = getGenericType( domainObject );
        if( genericType != null )
        {
            Set<String> classNames = getCustomImports( domainObject );
            for( String className : classNames )
            {
                if( className.endsWith( genericType ) )
                {
                    return className;
                }
            }
        }
        return null;
    }

    /**
     * Given a domain object that implements the Respository stereotype, this
     * method determines the generic type that should be used in its method signatures
     * and variables. For example, the WidgetDao probably works on Widgets. 
     * 
     * To be sure, this method finds the findById() method and uses its return value as
     * the generic type. If it doesn't find a findById() method then it tries to derive the
     * type from the name of the Repository, e.g. WidgetDao would return "Widget" and 
     * WidgetRepository would return "Widget". If the name does not end in Dao or Repository 
     * then it just returns null.
     *  
     * @param domainObject          The domain object for which to determine the generic type
     * 
     * @return                      The type to use in generic classes
     */
    public static String getGenericType( DomainObject domainObject )
    {
        if(  ! domainObject.containsStereotype( "Repository" ) )
        {
            // Only works for Repository objects
            return null;
        }

        // Find the findById() method
        for( Operation operation : domainObject.getOperations() )
        {
            if( operation.getName().equalsIgnoreCase( "findById" ) )
            {
                String genericType = operation.getReturnType();
                if( genericType.indexOf( '.' ) != -1 )
                {
                    return genericType.substring( genericType.lastIndexOf( '.' ) + 1 );
                }
                return genericType;
            }
        }

        // Fall back to the name
        if( domainObject.getName().endsWith( "Dao" ) )
        {
            String genericType = domainObject.getName();
            return genericType.substring( 0, genericType.length() - 3 );
        }
        else if( domainObject.getName().endsWith( "Repository" ) )
        {
            String genericType = domainObject.getName();
            return genericType.substring( 0, genericType.length() - 10 );
        }

        // Nope, nothing to go on ..
        return null;

    }

    /**
     * Searches through all return types and operation parameters and builds
     * a set of classes/interfaces that need to be imported for this class
     *
     * @param domainObject          The domain object to examine
     *
     * @return                      A set of all classes/interfaces upon which
     *                              this domain object depends
     */
    public static Set<String> getCustomImports( DomainObject domainObject )
    {
        Set<String> imports = new TreeSet<String>();

        // Iterate over all operations
        for( Operation operation : domainObject.getOperations() )
        {
            // Handle the return type
            String returnType = operation.getReturnType();
            if( returnType.indexOf( '.' ) != -1 )
            {
                imports.add( returnType );
            }

            // Handle the operation parameters
            for( OperationParameter parameter : operation.getParameters() )
            {
                String parameterType = parameter.getType();
                if( parameterType.indexOf( '.' ) != -1 )
                {
                    imports.add( parameterType );
                }
            }
        }

        // Return the set that we built
        return imports;
    }

    /**
     * Generates an HQL statement for the specified "findBy" operation. Returns a string
     * of the form:
     *
     *  "from com.foo.Widget widget where widget.id = ?", id
     *
     * @param fullyQualifiedDomainName  The fully qualified name of the domain object
     *                                  that owns this operation
     * @param operation                 The operation for which to generate the HQL
     *
     * @return                          A findByXXX HQL statement
     */
    public static String getHql( String fullyQualifiedDomainName, Operation operation )
    {
        // Build the HQL into a StringBuffer
        StringBuilder sb = new StringBuilder();

        // Build the domain field name, which will be used in the HQL statement,
        // For example, "widget" in: "from Widget widget where widget.id = ?"
        String domainFieldName = null;
        if( fullyQualifiedDomainName.indexOf( '.' ) != -1 )
        {
            domainFieldName = fullyQualifiedDomainName.substring( fullyQualifiedDomainName.lastIndexOf( '.' ) + 1 );
        }
        else
        {
            domainFieldName = fullyQualifiedDomainName;
        }
        domainFieldName = ConversionUtil.getFieldName( domainFieldName );

        String operationName = operation.getName();
        if( operationName.startsWith( "findBy" ) )
        {
            if( operationName.indexOf( "LessThan" ) != -1 )
            {
                // Search by a single field "findByFieldName"
                String fieldName = ConversionUtil.getFieldName( operationName.substring( 6, operationName.length() - 8 ) );
                sb.append( "\"from " ).append( fullyQualifiedDomainName );
                sb.append( " " ).append( domainFieldName );
                sb.append( " where " ).append( domainFieldName ).append( "." ).append( fieldName );
                sb.append( " < ?\"" );
            }
            else if( operationName.indexOf( "GreaterThan" ) != -1 )
            {
                // Search by a single field "findByFieldName"
                String fieldName = ConversionUtil.getFieldName( operationName.substring( 6, operationName.length() - 11 ) );
                sb.append( "\"from " ).append( fullyQualifiedDomainName );
                sb.append( " " ).append( domainFieldName );
                sb.append( " where " ).append( domainFieldName ).append( "." ).append( fieldName );
                sb.append( " > ?\"" );
            }
            else if( operationName.indexOf( "Between" ) != -1 )
            {
                // Search by a single field "findByFieldName"
                String fieldName = ConversionUtil.getFieldName( operationName.substring( 6, operationName.length() - 7 ) );
                sb.append( "\"from " ).append( fullyQualifiedDomainName );
                sb.append( " " ).append( domainFieldName );
                sb.append( " where " ).append( domainFieldName ).append( "." ).append( fieldName );
                sb.append( " > ? and " ).append( domainFieldName ).append( "." ).append( fieldName );
                sb.append( " < ?\"" );

            }
            else if( operationName.indexOf( "And" ) != -1 )
            {
            }
            else
            {
                // Search by a single field "findByFieldName"
                String fieldName = ConversionUtil.getFieldName( operationName.substring( 6 ) );
                sb.append( "\"from " ).append( fullyQualifiedDomainName );
                sb.append( " " ).append( domainFieldName );
                sb.append( " where " ).append( domainFieldName ).append( "." ).append( fieldName );
                sb.append( " = ?\"" );
            }

            // Add bind parameters
            for( OperationParameter parameter : operation.getParameters() )
            {
                sb.append( ", " ).append( parameter.getName() );
            }
        }

        // Return the string that we built
        return sb.toString();
    }

    public static String getLogMessage( Operation operation )
    {
        StringBuilder sb = new StringBuilder();

        sb.append( "\"Executing operation: " ).append( operation.getName() );
        if( operation.getParameters().size() > 0 )
        {
            sb.append( " with parameters: \" + " );
            for( OperationParameter parameter : operation.getParameters() )
            {
                sb.append( parameter.getName() );
                sb.append( " + \", \"  + " );
            }
            sb.delete( sb.length() - 10, sb.length() - 1 );
        }
        else
        {
            sb.append( "\"" );
        }

        return sb.toString();
    }
}
